# -*- coding: utf-8 -*-
import collections
import contextlib
import sys
import wave
import webrtcvad
import os


def read_wave(path):
    """Reads a .wav file.

    Takes the path, and returns (PCM audio data, sample rate).
    """
    with contextlib.closing(wave.open(path, 'rb')) as wf:
        num_channels = wf.getnchannels()
        assert num_channels == 1
        sample_width = wf.getsampwidth()
        assert sample_width == 2
        sample_rate = wf.getframerate()
        assert sample_rate in (8000, 16000, 32000, 48000)
        pcm_data = wf.readframes(wf.getnframes())
        return pcm_data, sample_rate


def write_wave(path, audio, sample_rate, isSave=True):
    """Writes a .wav file.

    Takes path, PCM audio data, and sample rate.
    """
    if not isSave:
        return
    with contextlib.closing(wave.open(path, 'wb')) as wf:
        wf.setnchannels(1)
        wf.setsampwidth(2)
        wf.setframerate(sample_rate)
        wf.writeframes(audio)


class Frame(object):
    """Represents a "frame" of audio data."""
    def __init__(self, bytes, timestamp, duration):
        self.bytes = bytes
        self.timestamp = timestamp
        self.duration = duration


def frame_generator(frame_duration_ms, audio, sample_rate):
    """Generates audio frames from PCM audio data.

    Takes the desired frame duration in milliseconds, the PCM data, and
    the sample rate.

    Yields Frames of the requested duration.
    """
    n = int(sample_rate * (frame_duration_ms / 1000.0) * 2)
    offset = 0
    timestamp = 0.0
    duration = (float(n) / sample_rate) / 2.0
    while offset + n < len(audio):
        yield Frame(audio[offset:offset + n], timestamp, duration)
        timestamp += duration
        offset += n


def vad_collector(sample_rate, frame_duration_ms, padding_duration_ms, vad,
                  frames):
    """Filters out non-voiced audio frames.

    Given a webrtcvad.Vad and a source of audio frames, yields only
    the voiced audio.

    Uses a padded, sliding window algorithm over the audio frames.
    When more than 90% of the frames in the window are voiced (as
    reported by the VAD), the collector triggers and begins yielding
    audio frames. Then the collector waits until 90% of the frames in
    the window are unvoiced to detrigger.

    The window is padded at the front and back to provide a small
    amount of silence or the beginnings/endings of speech around the
    voiced frames.

    Arguments:

    sample_rate - The audio sample rate, in Hz.
    frame_duration_ms - The frame duration in milliseconds.
    padding_duration_ms - The amount to pad the window, in milliseconds.
    vad - An instance of webrtcvad.Vad.
    frames - a source of audio frames (sequence or generator).

    Returns: A generator that yields PCM audio data.
    """
    num_padding_frames = int(padding_duration_ms / frame_duration_ms)
    # We use a deque for our sliding window/ring buffer.
    ring_buffer = collections.deque(maxlen=num_padding_frames)
    # We have two states: TRIGGERED and NOTTRIGGERED. We start in the
    # NOTTRIGGERED state.
    triggered = False

    voiced_frames = []
    for frame in frames:
        is_speech = vad.is_speech(frame.bytes, sample_rate)

        #         sys.stdout.write('1' if is_speech else '0')
        if not triggered:
            ring_buffer.append((frame, is_speech))
            num_voiced = len([f for f, speech in ring_buffer if speech])
            # If we're NOTTRIGGERED and more than 90% of the frames in
            # the ring buffer are voiced frames, then enter the
            # TRIGGERED state.
            if num_voiced > 0.9 * ring_buffer.maxlen:
                triggered = True
                triggered_time = ring_buffer[0][0].timestamp
                #                 sys.stdout.write('+(%s)' % (ring_buffer[0][0].timestamp,))
                # We want to yield all the audio we see from now until
                # we are NOTTRIGGERED, but we have to start with the
                # audio that's already in the ring buffer.
                for f, s in ring_buffer:
                    voiced_frames.append(f)
                ring_buffer.clear()
        else:
            # We're in the TRIGGERED state, so collect the audio data
            # and add it to the ring buffer.
            voiced_frames.append(frame)
            ring_buffer.append((frame, is_speech))
            num_unvoiced = len([f for f, speech in ring_buffer if not speech])
            # If more than 90% of the frames in the ring buffer are
            # unvoiced, then enter NOTTRIGGERED and yield whatever
            # audio we've collected.
            if num_unvoiced > 0.9 * ring_buffer.maxlen:
                #                 sys.stdout.write('-(%s)' % (frame.timestamp + frame.duration))
                triggered = False
                yield b''.join([f.bytes
                                for f in voiced_frames]), triggered_time
                triggered_time = 0
                ring_buffer.clear()
                voiced_frames = []


#    if triggered:
#         sys.stdout.write('-(%s)' % (frame.timestamp + frame.duration))
#     sys.stdout.write('\n')
# If we have any leftover voiced audio when we run out of input,
# yield it.
    if voiced_frames:
        yield b''.join([f.bytes for f in voiced_frames]), triggered_time


def vad(filename, mode=2, span=None, stride=None, chunk_dir=None):
    if chunk_dir and not os.path.exists(chunk_dir):
        os.makedirs(chunk_dir)
    chunk_path = os.path.join(chunk_dir if chunk_dir else '.',
                              'chunk-%01d.wav' % (mode, ))
    paths = [chunk_path]
    voiced_range = []
    voiced_duration = 0.0
    voiced_segment = b''
    audio, sample_rate = read_wave(filename)
    vad = webrtcvad.Vad(mode)
    # a frame is 30ms
    frames = list(frame_generator(30, audio, sample_rate))
    duration = len(frames) * 30 / 1000
    # a frame is 30ms & the queue is 200ms
    segments = vad_collector(sample_rate, 30, 200, vad, frames)
    for i, (segment, start_time) in enumerate(segments):
        path = os.path.join(chunk_dir if chunk_dir else '.',
                            'chunk-%002d-%01d.wav' % (i, mode))
        duration_ = len(segment) / sample_rate / 2
        paths.append(path)
        voiced_range.append([[start_time, start_time + duration_]])
        voiced_duration += duration_
        voiced_segment += segment
        if not span:
            write_wave(path, segment, sample_rate, bool(chunk_dir))

    if chunk_dir and len(paths) > 0:
        write_wave(chunk_path, voiced_segment, sample_rate, bool(chunk_dir))
        if span:
            if not stride:
                stride = span
            stride = max(stride, 1)
            span = max(span, 3)
            num = int((voiced_duration - span) / stride) + 1
            span = sample_rate * 2 * span
            stride = sample_rate * 2 * stride
            paths = [chunk_path]
            for i in range(num):
                path = os.path.join(chunk_dir, 'chunk-%002d-%01d.wav' % (
                    i,
                    mode,
                ))
                paths.append(path)
                write_wave(path, voiced_segment[i * stride:i * stride + span],
                           sample_rate)
    return paths if chunk_dir else [], voiced_range, voiced_duration, duration


def vad_robust(path, span=None, stride=None, chunk_dir=None):
    result = vad(path, span=span, stride=stride, mode=3, chunk_dir=chunk_dir)
    for x in [2, 1, 0]:
        mid = vad(path, span=span, stride=stride, mode=x, chunk_dir=chunk_dir)
        if len(mid[1]) > len(result[1]):
            result = mid
        else:
            if len(mid[1]) == len(result[1]):
                return mid
            break
    return result



from mmanalyser import config
import numpy as np
import logging


class HeartRateAnalyser:
    '''
    心率分析器
    '''
    def __init__(self):
        self.logger = logging.getLogger(self.__class__.__name__)

    def get_bpm_from_heart_rate(self, heart_rate):
        '''

        Args:
            heart_rate: [[70, 1000.0], [75, 2000.0], ...]

        Returns:
            list: [70, 71, ...] 平滑过的心率波动
        '''
        # 整合心率到每秒，考虑到帧率并不能与毫秒对齐
        count = 1
        bpm_res = []
        # 对心率结果平均duration作一次平均
        duration = config.heart_rate_duration
        bpm_tmps = []
        for x in heart_rate:
            if duration * count > x[1] >= duration * (count - 1):
                bpm_tmps.append(x[0])
            else:
                bpm_res.append(np.mean(bpm_tmps))
                bpm_tmps = [x[0]]
                count += 1
        if len(bpm_tmps) > 0:
            bpm_res.append(np.mean(bpm_tmps))
        length = len(bpm_res)
        bpm = []
        windows = config.heart_rate_windows  # 平滑窗口
        if windows > 1:
            for idx in range(length):
                if idx - windows < 0:
                    value = np.mean(bpm_res[:idx + windows + 1])
                elif idx + windows > length:
                    value = np.mean(bpm_res[idx - windows:])
                else:
                    value = np.mean(bpm_res[idx - windows:idx + windows + 1])
                bpm.append(value)
        else:
            bpm = bpm_res
        return bpm



voice_dBFS_diff=2
voice_dBFS_sample_interval=100
voice_dBFS_human_var_min=5
voice_dBFS_human_max=-10
voice_recognition_div_human_min=0.5
voice_ppl_max_length=15
voice_ppl_min_length=5
voice_modal_particles_pattern="啊|嗯|呃|呢|这个|那个"
voice_length_per_count_min=5
voice_length_per_count_max=25
voice_recognition_duration_per_pause_count_min=10
voice_recognition_duration_per_pause_count_max=50
voice_pause_duration_ratio_min=0
voice_pause_duration_ratio_max=0.3
voice_repeat_one_count_min=0
voice_repeat_one_count_max=5
voice_ppl_min=100
voice_ppl_max=200
skip_word4ppl=['我叫','岁','我是']
start_word4ppl=['然后','最后','接着']
text_clean_for_ppl="啊|嗯|呃|！|，|？|。"
text_clean_for_modal_particles="！|，|？|。"

KEY_FRAME_INTERVAL = 1


heart_rate_duration=1000
heart_rate_windows=1



from mmanalyser import config
import re
import numpy as np
from pydub import AudioSegment
from pypinyin import pinyin as get_pinyin, Style
from mmanalyser.utils.nlu_tools import NluTools
from mmanalyser.utils.wer import wer
from mmanalyser.utils.number_to_chinese import number_to_chinese
import logging


WORD_SEP = '#'
AUDIO_START_TIME = 500


class ExpressionAnalyser:
    '''
    表达能力分析器

    asr数据格式：{"pause_detection": [[3280, 3922], [7781, 8688]], "text_corrected":  [], "text":  [[[0, 1170], "领导你好，"],
                                     [[8940, 12250], "我在嘉兴，"], [[12250, 13490], "有两家造纸厂，"]]}
    '''
    def __init__(self, rationality_service_url=None):
        self.logger = logging.getLogger(self.__class__.__name__)
        self.rationality_service_url = rationality_service_url
        self.nlu = NluTools()
        self.ntc = number_to_chinese()

    def get_expression_score(self, asr, audio_path=None):
        """
        获取表达能力分数

        Args:
            asr: asr结果
            audio_path:  音频地址

        Returns:
             dict: {
            'talk_speed': float,        // 语速
            'audio_clear': bool,        //声音是否清晰
            'mandarin_score': float,    //普通话标准程度
            'expression_score': float,  //表达能力分数
            }
        """
        raw_text, recognition_duration, pause_moment = self.__get_text_information_from_asr(asr)
        text, text_length, particle_count, repeat_one_count, ranhou_count = self.__disfluency_detection(raw_text)
        mandarin_score = self.__parse_mandarin_score(text)
        talk_speed = self.__get_talk_speed(text_length, recognition_duration)
        audio_clear = self.__is_audio_clear(audio_path, text_length, recognition_duration, mandarin_score)
        express_score = self.__parse_expression_score(
            audio_clear, text_length, pause_moment, recognition_duration,
            particle_count, mandarin_score, repeat_one_count, ranhou_count) * 20
        return {
            'talk_speed': talk_speed,
            'audio_clear': audio_clear,
            'mandarin_score': mandarin_score*100 if mandarin_score != -1 else mandarin_score,
            'expression_score': express_score,
        }

    def get_talk_speed(self, asr):
        '''
        获取语速指标

        Args:
            asr: asr 结果
        Returns:
           float: 语速
        '''
        raw_text, recognition_duration, pause_moment = \
            self.__get_text_information_from_asr(asr)
        text, text_length, particle_count, repeat_one_count, ranhou_count = \
            self.__disfluency_detection(raw_text)
        talk_speed = self.__get_talk_speed(text_length, recognition_duration)
        return talk_speed

    def get_mandarin_score(self, asr, pinyin=None):
        '''
        获取普通话标准程度

        Args:
            asr: asr结果
            pinyin: 拼音识别结果（"wo2 ai2 ni3"）
        Returns:
            float: 普通话标准程度
        '''
        def func(x):
            if x > 1.5:
                return  0.15/x
            elif x < 0.5:
                return 0.05*(0.5-x)/0.5 + 0.95
            else:
                return (1.5-x)/1.5*0.8+0.15

        if pinyin:
            text = self.ntc.parse(''.join([x[1] for x in asr['text']]))
            text = re.sub("！|，|？|。| |,", '', text)
            text = ' '.join([x[0] for x in get_pinyin(str(text), style=Style.TONE3)])
            return func(wer(text, pinyin)) * 100

        raw_text, recognition_duration, pause_moment = \
            self.__get_text_information_from_asr(asr)
        text, text_length, particle_count, repeat_one_count, ranhou_count = \
            self.__disfluency_detection(raw_text)
        mandarin_score = self.__parse_mandarin_score(text)
        return mandarin_score*100 if mandarin_score != -1 else mandarin_score

    def get_audio_clear(self, asr, audio_path):
        '''
        获取声音是否清晰

        Args:
            asr: asr结果
            audio_path: 音频地址

        Returns:
            bool: 清晰返回true 不清晰返回false
        '''
        raw_text, recognition_duration, pause_moment = \
            self.__get_text_information_from_asr(asr)
        text, text_length, particle_count, repeat_one_count, ranhou_count = \
            self.__disfluency_detection(raw_text)
        mandarin_score = self.__parse_mandarin_score(text)
        audio_clear = self.__is_audio_clear(audio_path, text_length, recognition_duration, mandarin_score)
        return audio_clear

    def __get_talk_speed(self, text_length, recognition_duration):
        self.logger.debug("get talk speed")
        return text_length / recognition_duration

    def __is_audio_clear(self, audio_path, length, recognition_duration, mandarin_score):
        self.logger.debug("get audio clear")
        if mandarin_score == -1:
            mandarin_score = 0.5

        flag = (mandarin_score != 0 and recognition_duration > 0 and (length / recognition_duration > 0.75 or
                                                                      (length / recognition_duration < 0.75 and mandarin_score > 0.5)))
        if audio_path is not None:
            dBFS_var, dBFS_mean = self.__get_dBFS_information(audio_path)
            return bool(dBFS_mean < config.voice_dBFS_human_max and
                    dBFS_var > config.voice_dBFS_human_var_min and flag)
        else:
            return flag

    def __parse_mandarin_score(self, text):
        self.logger.debug("parse mandarin score")
        if self.rationality_service_url is None:
            return -1

        sentences = self.__preprocess_text_for_ppl(text)
        if not sentences:
            return 0
        ppls = self.nlu.get_rationality(sentences, self.rationality_service_url)
        # 句子合理性 0~1分
        ppl_mark = 0
        for ppl in ppls:
            if ppl < config.voice_ppl_min:
                ppl_mark += 1
            elif ppl < config.voice_ppl_max:
                voice_ppl_range = config.voice_ppl_max - config.voice_ppl_min
                voice_ppl_diff = ppl - config.voice_ppl_min
                ppl_mark += 1 - voice_ppl_diff / voice_ppl_range
        return ppl_mark / (len(ppls)+0.2) if ppls else -1

    def __parse_expression_score(
            self,
            audio_clear,
            text_length,
            pause_moment,
            recognition_duration,
            particle_count,
            mandarin_score,
            repeat_one_count,
            ranhou_count):
        self.logger.debug("get expression score")
        if not audio_clear:
            return 0

        if mandarin_score == -1:
            mandarin_score = 0.5

        if len(pause_moment) == 0:
            recognition_duration_per_pause_count = config.voice_recognition_duration_per_pause_count_max
        else:
            recognition_duration_per_pause_count = text_length / \
                len(pause_moment)

        recognition_duration_per_pause_count_score = self.__min_max_scaler(
            recognition_duration_per_pause_count,
            config.voice_recognition_duration_per_pause_count_min,
            config.voice_recognition_duration_per_pause_count_max)
        pause_duration_ratio = sum(pause_moment) / recognition_duration if recognition_duration > 0 else 1
        pause_duration_ratio_score = self.__min_max_scaler(
            pause_duration_ratio,
            config.voice_pause_duration_ratio_min,
            config.voice_pause_duration_ratio_max,
            negative=True)

        # 语气词 0~1分
        if particle_count == 0:
            length_per_count = config.voice_length_per_count_max
        else:
            length_per_count = text_length / particle_count
        length_per_count_score = self.__min_max_scaler(
            length_per_count, config.voice_length_per_count_min,
            config.voice_length_per_count_max)
        repeat_one_count_score = self.__min_max_scaler(
            repeat_one_count,
            config.voice_repeat_one_count_min,
            config.voice_repeat_one_count_max,
            negative=True)

        ranhou_count_per_length = ranhou_count / text_length
        ranhou_count_per_length_score = self.__min_max_scaler(
            ranhou_count_per_length, 0.02, 0.06, negative=True)

        decay_factor = 0.8 if pause_duration_ratio_score == 0 or \
            recognition_duration_per_pause_count_score == 0 or \
            length_per_count_score == 0 or \
            repeat_one_count_score == 0 else 1

        return (0.75 * pause_duration_ratio_score + 0.75 *
                recognition_duration_per_pause_count_score + 1.5 * length_per_count_score +
                1.5 * repeat_one_count_score + ranhou_count_per_length_score * 0.5) * decay_factor * (0.85 + 0.1 * mandarin_score/100)

    def __get_text_information_from_asr(self, asr):
        recognition_duration = sum(
            [int(x[0][1]) - int(x[0][0]) for x in asr['text']]) / 1000
        # 停顿
        pause_moment = [(x[0]-x[1])/1000 for x in asr["pause_detection"] if x[1] - x[0] > 500]
        text = ''.join([x[1] for x in asr['text']])
        return text, recognition_duration, pause_moment

    def __disfluency_detection(self, text):
        # 语气词
        particle_count = len(
            re.findall(config.voice_modal_particles_pattern,
                       re.sub(config.text_clean_for_modal_particles, '', text)))

        # 文本清理
        text = re.sub(config.text_clean_for_ppl, WORD_SEP, text)

        _text = text.replace(WORD_SEP, '')
        repeat_one_count = len(self.__get_repeat_record_by_offset(_text, 1))
        ranhou_count = len(re.findall('然后', _text))
        text_length = len(_text)
        return text, text_length, particle_count, repeat_one_count, ranhou_count

    def __get_dBFS_information(self, audio):
        sound = AudioSegment.from_file(audio, "mp3")
        dBFS = []
        for x in range(AUDIO_START_TIME, len(sound),
                       config.voice_dBFS_sample_interval):
            dbfs = sound[x:x + config.voice_dBFS_sample_interval].dBFS
            if dbfs != float("-inf"):
                dBFS.append(dbfs)
        return np.var(dBFS), np.mean(dBFS)

    def __preprocess_text_for_ppl(self, text):
        MAX_SENTENCE_LEN = 5
        MIN_SENTENCE_LEN = 2

        def need_concate_sentence(word, sentence):
            not_has_start_words = all(
                [prefix not in word for prefix in config.start_word4ppl])
            return not_has_start_words and len(sentence) <= MAX_SENTENCE_LEN

        def has_skip_word(sentence):
            return any(
                [skip_word in sentence for skip_word in config.skip_word4ppl])
        words = text.split(WORD_SEP)
        sentences = []
        sentence = ""
        for word in words:
            if need_concate_sentence(word, sentence):
                sentence += word
            else:
                if len(sentence) > MIN_SENTENCE_LEN:
                    sentences.append(sentence)
                sentence = word
        if len(sentence) > MIN_SENTENCE_LEN:
            sentences.append(sentence)
        _sentences = [sent for sent in sentences if not has_skip_word(sent)]
        if len(_sentences) == 0:
            _sentences = sentences
        return _sentences

    def __min_max_scaler(self, value, min_value, max_value, negative=False):
        score = 0
        if value > max_value:
            score = 1
        elif value > min_value:
            diff = value - min_value
            score += diff / (max_value - min_value)
        return score if not negative else 1 - score


    def __get_repeat_record_by_offset(self, text, offset=1):
        assert offset > 0
        length = len(text)
        repeat_record = []
        word = text[0:offset] if length > 0 else ''
        i = offset * 2
        count = 1
        while i < length + 1:
            if word == text[i - offset:i]:
                count += 1
            else:
                if count > 1:
                    repeat_record.append(count)
                i = i - offset + 1
                count = 1
                word = text[i - offset:i]
            i = i + offset
        if count > 1:
            repeat_record.append(count)
        return repeat_record


if __name__ == '__main__':
    ea = ExpressionAnalyser()
    fake_asr = {
        "pause_detection": [[3280, 3922], [7781, 8688]],  # 停顿检测结果
        "text_corrected":  [[[0, 1170], "领导你好，"], [[1170, 2370], "我叫秦嘉涵，"], [[2370, 5530], "今年26岁，"], [[5530, 8040], "是浙江龙游人，"], [[8040, 8940], "我，"], [[8940, 12250], "我在龙游，"], [[12250, 13490], "有两家造纸厂，"], [[13490, 19330], "有4年的维修工禁令。"]],
        "text":  [[[0, 1170], "领导你好，"], [[1170, 2370], "我叫秦嘉涵，"], [[2370, 5530], "今年26岁，"], [[5530, 8040], "是浙江龙游人，"], [[8040, 8940], "我，"], [[8940, 12250], "我在龙游，"], [[12250, 13490], "有两家造纸厂，"], [[13490, 19330], "有4年的维修工禁令。"]]
    }
    fake_pinyin = ' '.join([x[0] for x in get_pinyin("领导你好", style=Style.TONE3)])
    # print(ea.get_expression_score(asr=fake_asr, verbose=True))
    print(ea.get_mandarin_score(asr=fake_asr, pinyin=fake_pinyin))









import pandas as pd
import numpy as np
import logging

class EmotionType:
    ANGER = "anger"
    NEUTRAL = "neutral"
    DISGUST = "disgust"
    FEAR = "fear"
    HAPPINESS = "happiness"
    SADNESS = "sadness"
    SURPRISE = "surprise"


class EmotionAnalyser:
    '''
    情绪分析器
    '''
    EMOTIONS = [EmotionType.ANGER, EmotionType.NEUTRAL,
                EmotionType.DISGUST, EmotionType.FEAR,
                EmotionType.HAPPINESS, EmotionType.SADNESS,
                EmotionType.SURPRISE]

    def __init__(self):
        self.logger = logging.getLogger(self.__class__.__name__)

    def merge_emotion(self, audio_emotion, frame_emotion):
        pass

    def get_emotion_global_distribution(self, emotion):
        """

        获取情绪的分布

        Args:
            emotion: [{"anger": 0.1, "neutral": 0.1, "disgust": 0.1, "fear": 0.005670338961718315, "happiness": 0.2 "sadness": 0.1, "surprise": 0.2}, ...]

        Returns:
            dict: distribution -> {'anger': 0.04678370220399322, 'neutral': 0.04147910314122111, 'disgust': 0.05076599764308418, 'fear': 0.012459961661656426, 'happiness': 0.3805726751609995, 'sadness': 0.001341534333740512, 'surprise': 0.46659702585530505}
        """

        emotion_df = self.__construct_emotion_df(emotion)
        distribution = self.__calculate_distribution(emotion_df)
        return distribution

    def get_emotion_from_video(self, video_path):
        pass

    def get_fluctuation_from_emotion(self, emotion):
        """

        获取情绪的波动

        Args:
            emotion: [{"anger": 0.1, "neutral": 0.1, "disgust": 0.1, "fear": 0.005670338961718315, "happiness": 0.2 "sadness": 0.1, "surprise": 0.2}, ...]

        Returns:
            list: fluction -> [0.2625822317841726, 0.286318327656083, 0.19226654361134174, 0.26339393328225225, 0.3021995866171042, 0.2843571171657221, ...]
        """

        emotion_df = self.__construct_emotion_df(emotion)
        distribution = self.__calculate_distribution(emotion_df)
        fluctuation = self.__calculate_fluctuation(emotion_df, distribution)
        return fluctuation

    def parse(self, emotion):
        '''
        情绪整体分析，返回情绪数据，情绪总体分布，情绪波动

        Args:
            emotion: [{"anger": 0.1, "neutral": 0.1, "disgust": 0.1, "fear": 0.005670338961718315, "happiness": 0.2 "sadness": 0.1, "surprise": 0.2}, ...]

        Returns:
            list: data ->[[2.639, 1.415, 45.941, 3.918, 34.431, 0.159, 11.497], [0.685, 18.661, ... ] ... ]  顺序为['anger', 'neutral', 'disgust', 'fear', 'happiness', 'sadness', 'surprise']
            dict: distribution
            list: fluction
        '''

        emotion_df = self.__construct_emotion_df(emotion)
        distribution = self.__calculate_distribution(emotion_df)
        fluctuation = self.__calculate_fluctuation(emotion_df, distribution)
        return emotion_df.round(decimals=3).values.tolist(), distribution, fluctuation

    def __construct_emotion_df(self, emotion):
        self.logger.debug("construct emotion dataframes")
        emotion_df = pd.DataFrame(emotion)
        # 设置表头
        emotion_df = emotion_df[self.EMOTIONS]
        # 设置需要插值的位置
        emotion_df[emotion_df.sum(axis=1) == 0] = np.nan
        # 插值每一列中nan的数值
        emotion_df = emotion_df.interpolate(
                method='linear', limit_direction='both', axis=0)
        emotion_df = emotion_df.fillna(0)
        return emotion_df

    def __calculate_distribution(self, emotion_df):
        self.logger.debug("calculate the emotion distribution")
        each_emotion_sum = {e: sum(emotion_df[e]) for e in emotion_df.keys()}
        emotion_sum = sum(each_emotion_sum.values())
        if emotion_sum == 0:
            return {}
        return {k: v * 1.0 / emotion_sum for k, v in each_emotion_sum.items()}

    def __calculate_fluctuation(self, emotion_df, distribution):
        self.logger.debug("calculate the emotion fluction")
        if not distribution:
            return []
        fluctuation = []
        average_emotion = distribution[EmotionType.HAPPINESS] -\
                            distribution[EmotionType.SADNESS]
        for index, row in emotion_df.iterrows():
            emotion_sum = sum(row[e] for e in [EmotionType.SADNESS,
                                               EmotionType.NEUTRAL,
                                               EmotionType.HAPPINESS])
            if emotion_sum == 0:
                fluctuation.append(0)
            else:
                negative_score = row[EmotionType.SADNESS] / emotion_sum
                positive_score = row[EmotionType.HAPPINESS] / emotion_sum
                fluctuation.append(
                        (positive_score - negative_score - average_emotion) / 2)
        return fluctuation


if __name__ == "__main__":
    fake_emotion = [{'anger': 0.001, 'neutral': 0.852, 'disgust': 0.028, 'fear': 0.002,
             'happiness': 0.058, 'sadness': 0.011, 'surprise': 0.048},
            {
                'anger': 0.001, 'neutral': 0.852, 'disgust': 0.028, 'fear': 0.002, 'happiness': 0.058, 'sadness': 0.011,
                'surprise': 0.048
            }]
    ea = EmotionAnalyser()
    print(ea.EMOTIONS)
    print(ea.parse(fake_emotion))




import json
import logging
import requests


class NluTools(object):
    def __init__(self):
        self.logger = logging.getLogger(self.__class__.__name__)

    def httpcall(self, url, req):
        try:
            response = requests.post(
                    url, json.dumps(req).encode("utf8"), timeout=10)
        except Exception as e:
            self.logger.exception("nlu error[%s]", e)
            return None
        if response.status_code != 200:
            return None
        return json.loads(response.text)

    def get_rationality(self, array, rationality_service_url):
        req = {
            "header": {'Content-Type': 'application/json'},
            "traffic_paramsDict": {
                "text": array,
                "prob": False,
                "strategy": "seq"
            }
        }

        result = self.httpcall(rationality_service_url, req)
        if result is None:
            return []

        if result['status']:
            return result['result']['ppl']
        else:
            self.logger.error("Get Rationality Failed - Error Msg: %s", result['info'])
            return []




# -*- coding: utf-8 -*-
from mmanalyser import config
import numpy as np
from pydub import AudioSegment
from mmanalyser.utils.vad import vad_robust
from pyAudioAnalysis import audioTrainTest as aT
import logging
import os

WORD_SEP = '#'
AUDIO_START_TIME = 500


class AudioAnalyser:
    '''
    音频分析器

    '''
    def __init__(self, rationality_service_url=None):
        self.logger = logging.getLogger(self.__class__.__name__)

    def is_human_voice(self, audio_path):
        paths, voiced_range, voiced_duration, duration = vad_robust(audio_path, span=3, chunk_dir='temp/')
        if voiced_duration >= 4:
            label = (float(np.mean([aT.file_classification(x, os.path.join(os.path.dirname(os.path.dirname(__file__)),'mmanalyser', 'data', "gbdt"), "gradientboosting")[1][0] for x in paths[1:]])) > 0.5)
        else:
            label = False
        return label

    def delete_silence_snippet(self, audio_path, chunk_dir='temp/'):
        if chunk_dir is None:
            chunk_dir = 'temp/'
        return vad_robust(audio_path, chunk_dir='temp/')






# -*- coding: UTF-8 -*-
# !/usr/bin/python

import random
import re
# number_to_chinese, 单位-数字
num_dict = {
    0: "零",
    1: "一",
    2: "二",
    3: "三",
    4: "四",
    5: "五",
    6: "六",
    7: "七",
    8: "八",
    9: "九"
}
unit_map = [["", "十", "百", "千"], ["万", "十万", "百万", "千万"],
            ["亿", "十亿", "百亿", "千亿"], ["兆", "十兆", "百兆", "千兆"]]
unit_step = ["万", "亿", "兆"]


class number_to_chinese():
    """
       参考: https://github.com/tyong920/a2c
    """
    def __init__(self):
        self.result = ""

    def number_to_str_10000(self, data_str):
        """一万以内的数转成大写"""
        res = []
        count = 0
        # 倒转
        str_rev = reversed(
            data_str
        )  # seq -- 要转换的序列，可以是 tuple, string, list 或 range。返回一个反转的迭代器。
        for i in str_rev:
            if i is not "0":
                count_cos = count // 4  # 行
                count_col = count % 4  # 列
                res.append(unit_map[count_cos][count_col])
                res.append(num_dict[int(i)])
                count += 1
            else:
                count += 1
                if not res:
                    res.append("零")
                elif res[-1] is not "零":
                    res.append("零")
        # 再次倒序，这次变为正序了
        res.reverse()
        # 去掉"一十零"这样整数的“零”
        if res[-1] is "零" and len(res) is not 1:
            res.pop()
        return "".join(res)

    def number_to_str(self, data):
        """分段转化"""
        assert type(data) == float or int
        data_str = str(data)
        len_data = len(str(data_str))
        count_cos = len_data // 4  # 行
        count_col = len_data - count_cos * 4  # 列
        if count_col > 0:
            count_cos += 1

        res = ""
        for i in range(count_cos):
            if i == 0:
                data_in = data_str[-4:]
            elif i == count_cos - 1 and count_col > 0:
                data_in = data_str[:count_col]
            else:
                data_in = data_str[-(i + 1) * 4:-(i * 4)]
            res_ = self.number_to_str_10000(data_in)
            res = res_ + unit_map[i][0] + res
        return res

    def decimal_chinese(self, data):
        assert type(data) == float or int
        data_str = str(data)
        if "." not in data_str:
            res = self.number_to_str(data_str)
        else:
            data_str_split = data_str.split(".")
            if len(data_str_split) is 2:
                res_start = self.number_to_str(data_str_split[0])
                res_end = "".join(
                    [num_dict[int(number)] for number in data_str_split[1]])
                res = res_start + random.sample(["点", "."], 1)[0] + res_end
            else:
                res = str(data)
        return res

    def parse(self, text):
        text_ = ''
        it = re.finditer(r"\d+\.?\d*", text)
        x, y = 0, 0
        for match in it:
            x, y_ = match.span()
            text_ += text[y:x]
            text_ += self.decimal_chinese(match.group())
            y = y_
        text_ += text[y:]
        return text_
